@@ -308,3 +308,119 @@ class CausalTreeLearn(CausalTree):
         check_dir(filename)
         with open(filename, "wb") as file:
             pkl.dump(self, file)
+
+    def _eval_fast_honest(self, train_x, train_y, train_t, val_x, val_y, val_t, unique_vals, col, est_x, est_y, est_t):
+
+        min_size = self.min_size
+        val_size = self.val_split * self.min_size if self.val_split * self.min_size > 2 else 2
+
+        train_col_x = train_x[:, col]
+        val_col_x = val_x[:, col]
+        est_col_x = est_x[:, col]
+
+        xx = np.tile(train_col_x, (unique_vals.shape[0], 1))
+        yy = np.tile(train_y, (unique_vals.shape[0], 1))
+        tt = np.tile(train_t, (unique_vals.shape[0], 1))
+
+        val_xx = np.tile(val_col_x, (unique_vals.shape[0], 1))
+        val_yy = np.tile(val_y, (unique_vals.shape[0], 1))
+        val_tt = np.tile(val_t, (unique_vals.shape[0], 1))
+
+        est_xx = np.tile(est_col_x, (unique_vals.shape[0], 1))
+        est_yy = np.tile(est_y, (unique_vals.shape[0], 1))
+        est_tt = np.tile(est_t, (unique_vals.shape[0], 1))
+
+        idx_x = np.transpose(np.transpose(xx) >= unique_vals)
+        val_idx_x = np.transpose(np.transpose(val_xx) >= unique_vals)
+        est_idx_x = np.transpose(np.transpose(est_xx) >= unique_vals)
+
+        train_denom_treated_upper = idx_x * (tt == 1)
+        train_denom_control_upper = idx_x * (tt == 0)
+        train_treat_nums_upper = np.sum(train_denom_treated_upper, axis=1)
+        train_control_nums_upper = np.sum(train_denom_control_upper, axis=1)
+        train_denom_treated_lower = ~idx_x * (tt == 1)
+        train_denom_control_lower = ~idx_x * (tt == 0)
+        train_treat_nums_lower = np.sum(train_denom_treated_lower, axis=1)
+        train_control_nums_lower = np.sum(train_denom_control_lower, axis=1)
+
+        val_denom_treated_upper = val_idx_x * (val_tt == 1)
+        val_denom_control_upper = val_idx_x * (val_tt == 0)
+        val_treat_nums_upper = np.sum(val_denom_treated_upper, axis=1)
+        val_control_nums_upper = np.sum(val_denom_control_upper, axis=1)
+        val_denom_treated_lower = ~val_idx_x * (val_tt == 1)
+        val_denom_control_lower = ~val_idx_x * (val_tt == 0)
+        val_treat_nums_lower = np.sum(val_denom_treated_lower, axis=1)
+        val_control_nums_lower = np.sum(val_denom_control_lower, axis=1)
+
+        est_denom_treated_upper = est_idx_x * (est_tt == 1)
+        est_denom_control_upper = est_idx_x * (est_tt == 0)
+        est_treat_nums_upper = np.sum(est_denom_treated_upper, axis=1)
+        est_control_nums_upper = np.sum(est_denom_control_upper, axis=1)
+        est_denom_treated_lower = ~est_idx_x * (est_tt == 1)
+        est_denom_control_lower = ~est_idx_x * (est_tt == 0)
+        est_treat_nums_lower = np.sum(est_denom_treated_lower, axis=1)
+        est_control_nums_lower = np.sum(est_denom_control_lower, axis=1)
+
+        split_upper_check = np.logical_and(train_treat_nums_upper >= min_size, train_control_nums_upper >= min_size)
+        split_lower_check = np.logical_and(train_treat_nums_lower >= min_size, train_control_nums_lower >= min_size)
+        val_split_upper_check = np.logical_and(val_treat_nums_upper >= val_size, val_control_nums_upper >= val_size)
+        val_split_lower_check = np.logical_and(val_treat_nums_lower >= val_size, val_control_nums_lower >= val_size)
+        est_split_upper_check = np.logical_and(est_treat_nums_upper >= min_size, est_control_nums_upper >= min_size)
+        est_split_lower_check = np.logical_and(est_treat_nums_lower >= min_size, est_control_nums_lower >= min_size)
+        train_split_check = np.logical_and(split_upper_check, split_lower_check)
+        val_split_check = np.logical_and(val_split_upper_check, val_split_lower_check)
+        est_split_check = np.logical_and(est_split_upper_check, est_split_lower_check)
+        train_val_check = np.logical_and(train_split_check, val_split_check)
+        train_est_check = np.logical_and(train_val_check, est_split_check)
+        min_size_idx = np.where(train_est_check)
+
+        if len(min_size_idx[0]) < 1:
+            return -np.inf, -np.inf, -np.inf, -np.inf
+
+        unique_vals = unique_vals[min_size_idx]
+
+        # idx_x = idx_x[min_size_idx]
+        yy = yy[min_size_idx]
+        # tt = tt[min_size_idx]
+        train_denom_treated_upper = train_denom_treated_upper[min_size_idx]
+        train_denom_control_upper = train_denom_control_upper[min_size_idx]
+        train_denom_treated_lower = train_denom_treated_lower[min_size_idx]
+        train_denom_control_lower = train_denom_control_lower[min_size_idx]
+
+        # val_idx_x = val_idx_x[min_size_idx]
+        val_yy = val_yy[min_size_idx]
+        # val_tt = val_tt[min_size_idx]
+        val_denom_treated_upper = val_denom_treated_upper[min_size_idx]
+        val_denom_control_upper = val_denom_control_upper[min_size_idx]
+        val_denom_treated_lower = val_denom_treated_lower[min_size_idx]
+        val_denom_control_lower = val_denom_control_lower[min_size_idx]
+
+        train_um1 = np.sum(yy * train_denom_treated_upper, axis=1) / np.sum(train_denom_treated_upper, axis=1)
+        train_um0 = np.sum(yy * train_denom_control_upper, axis=1) / np.sum(train_denom_control_upper, axis=1)
+        train_upper_effect = train_um1 - train_um0
+        train_lm1 = np.sum(yy * train_denom_treated_lower, axis=1) / np.sum(train_denom_treated_lower, axis=1)
+        train_lm0 = np.sum(yy * train_denom_control_lower, axis=1) / np.sum(train_denom_control_lower, axis=1)
+        train_lower_effect = train_lm1 - train_lm0
+
+        val_um1 = np.sum(val_yy * val_denom_treated_upper, axis=1) / np.sum(val_denom_treated_upper, axis=1)
+        val_um0 = np.sum(val_yy * val_denom_control_upper, axis=1) / np.sum(val_denom_control_upper, axis=1)
+        val_upper_effect = val_um1 - val_um0
+        val_lm1 = np.sum(val_yy * val_denom_treated_lower, axis=1) / np.sum(val_denom_treated_lower, axis=1)
+        val_lm0 = np.sum(val_yy * val_denom_control_lower, axis=1) / np.sum(val_denom_control_lower, axis=1)
+        val_lower_effect = val_lm1 - val_lm0
+
+        train_upper_mse = train_upper_effect ** 2
+        upper_cost = np.abs(train_upper_effect - val_upper_effect)
+        upper_obj = train_upper_mse - upper_cost
+        train_lower_mse = train_lower_effect ** 2
+        lower_cost = np.abs(train_lower_effect - val_lower_effect)
+        lower_obj = train_lower_mse - lower_cost
+
+        split_obj = upper_obj + lower_obj
+        best_obj_idx = np.argmax(split_obj)
+        best_upper_obj = upper_obj[best_obj_idx]
+        best_lower_obj = lower_obj[best_obj_idx]
+        best_split_obj = split_obj[best_obj_idx]
+        val = unique_vals[best_obj_idx]
+
+        return best_split_obj, best_upper_obj, best_lower_obj, val
\ No newline at end of file
